@@ -24,27 +24,6 @@ import ConfigParser
 import base64
 import zlib
 
-# global flag for keyring availabilty
-keyring_available = False
-# Determine if keyring module and an implementation is available for secure password storage
-
-try:
-    # Linux systems should use keyring only if it comes with the distro, otherwise chances are small
-    # that keyring works at all
-    if platform.system() == "Linux":
-        # keyring and secretstorage have to be importable
-        import keyring, secretstorage
-        if ("SecretService") in dir(keyring.backends) and not (keyring.get_keyring() is None):
-			keyring_available = True
-    else:
-        # nagstamon.spec always have to match module path
-        import Nagstamon.thirdparty.keyring as keyring
-        keyring_available = not (keyring.get_keyring() is None)
-except:
-    import traceback
-    traceback.print_exc(file=sys.stdout)
-    keyring_available = False
-
 
 class Config(object):
     """
@@ -63,7 +42,7 @@ class Config(object):
         self.show_tooltips = True
         self.highlight_new_events = True
         self.default_sort_field = "Status"
-        self.default_sort_order = "Ascending"
+        self.default_sort_order = "Descending"
         self.filter_all_down_hosts = False
         self.filter_all_unreachable_hosts = False
         self.filter_all_flapping_hosts = False
@@ -171,6 +150,9 @@ class Config(object):
         self.defaults_downtime_type_fixed = True
         self.defaults_downtime_type_flexible = False
         self.converted_from_single_configfile = False
+        # internal flag to determine if keyring is available at all - defaults to False
+        # use_system_keyring is checked and defined some lines later after config file was read
+        self.keyring_available = False
 
         # Special FX
         # Centreon
@@ -185,6 +167,7 @@ class Config(object):
         # try to use a given config file - there must be one given
         # if sys.argv is larger than 1
         if len(sys.argv) > 1:
+            # MacOSX related -psn argument by launchd
             if sys.argv[1].find("-psn") != -1:
                 # new configdir approach
                 self.configdir = os.path.expanduser('~') + os.sep + ".nagstamon"
@@ -228,13 +211,35 @@ class Config(object):
                 config = ConfigParser.ConfigParser(allow_no_value=True)
             config.read(self.configfile)
 
+            # temporary dict for string-to-bool-conversion
+            BOOLPOOL = {"False": False, "True": True}
+
             # go through all sections of the conf file
             for section in config.sections():
                 # go through all items of each sections (in fact there is only on
                 # section which has to be there to comply to the .INI file standard
                 for i in config.items(section):
                     # create a key of every config item with its appropriate value
-                    object.__setattr__(self, i[0], i[1])
+                    # check first if it is a bool value and convert string if it is
+                    if i[1] in BOOLPOOL:
+                        object.__setattr__(self, i[0], BOOLPOOL[i[1]])
+                    else:
+                        object.__setattr__(self, i[0], i[1])
+                        
+            # because the switch from Nagstamon 1.0 to 1.0.1 brings the use_system_keyring property
+            # and all the thousands 1.0 installations do not know it yet it will be more comfortable
+            # for most of the Windows users if it is only defined as False after it was checked
+            # from config file
+            if not self.__dict__.has_key("use_system_keyring"):
+                if self.unconfigured == True:
+                    # an unconfigured system should start with no keyring to prevent crashes
+                    self.use_system_keyring = False
+                else:
+                    # a configured system seemed to be able to run and thus use system keyring
+                    if platform.system() in ["Windows", "Darwin"]:
+                        self.use_system_keyring = True
+                    else:
+                        self.use_system_keyring = self.KeyringAvailable()
 
             # reset self.configdir to temporarily saved value in case it differs from
             # the one read from configfile and so it would fail to save next time
@@ -335,6 +340,8 @@ class Config(object):
         """
         load servers config - special treatment because of obfuscated passwords
         """
+        self.keyring_available = self.KeyringAvailable()
+
         servers = self.LoadMultipleConfig("servers", "server", "Server")
         # deobfuscate username + password inside a try-except loop
         # if entries have not been obfuscated yet this action should raise an error
@@ -348,7 +355,12 @@ class Config(object):
                 # passwords for monitor server and proxy
                 if servers[server].save_password == "False":
                     servers[server].password = ""
-                elif keyring_available:
+                elif self.keyring_available and self.use_system_keyring:
+                    # necessary to import on-the-fly due to possible Windows crashes
+                    try:
+                        import keyring
+                    except:
+                        import Nagstamon.thirdparty.keyring as keyring
                     password = keyring.get_password("Nagstamon", "@".join((servers[server].username,
                                                                            servers[server].monitor_url))) or ""
                     if password == "":
@@ -359,7 +371,12 @@ class Config(object):
                 elif servers[server].password != "":
                     servers[server].password = self.DeObfuscate(servers[server].password)
                 # proxy password
-                if keyring_available:
+                if self.keyring_available and self.use_system_keyring:
+                    # necessary to import on-the-fly due to possible Windows crashes
+                    try:
+                        import keyring
+                    except:
+                        import Nagstamon.thirdparty.keyring as keyring
                     proxy_password = keyring.get_password("Nagstamon", "@".join(("proxy",
                                                                                  servers[server].proxy_username,
                                                                                  servers[server].proxy_address))) or ""
@@ -466,6 +483,21 @@ class Config(object):
                 if not option in ["servers", "actions"]:
                     config.set("Nagstamon", option, self.__dict__[option])
 
+            # because the switch from Nagstamon 1.0 to 1.0.1 brings the use_system_keyring property
+            # and all the thousands 1.0 installations do not know it yet it will be more comfortable
+            # for most of the Windows users if it is only defined as False after it was checked
+            # from config file
+            if not self.__dict__.has_key("use_system_keyring"):
+                if self.unconfigured == True:
+                    # an unconfigured system should start with no keyring to prevent crashes
+                    self.use_system_keyring = False
+                else:
+                    # a configured system seemed to be able to run and thus use system keyring
+                    if platform.system() in ["Windows", "Darwin"]:
+                        self.use_system_keyring = True
+                    else:
+                        self.use_system_keyring = self.KeyringAvailable()
+
             # save servers dict
             self.SaveMultipleConfig("servers", "server")
 
@@ -483,7 +515,11 @@ class Config(object):
             f = open(os.path.normpath(self.configfile), "w")
             config.write(f)
             f.close()
-        except:
+        except Exception, err:
+            print err
+            import traceback
+            traceback.print_exc(file=sys.stdout)
+
             # debug
             if str(self.debug_mode) == "True":
                 if server != None:
@@ -502,59 +538,86 @@ class Config(object):
         "multiple" means that multiple confs for actions or servers are loaded,
         not just one like for e.g. sound file
         """
-        try:
-            # one section for each setting
-            for s in self.__dict__[settingsdir]:
-                # depending on python version allow_no_value is allowed or not
-                if sys.version_info[0] < 3 and sys.version_info[1] < 7:
-                    config = ConfigParser.ConfigParser()
-                else:
-                    config = ConfigParser.ConfigParser(allow_no_value=True)
-                config.add_section(setting + "_" + s)
-                for option in self.__dict__[settingsdir][s].__dict__:
-                    # obfuscate certain entries in config file - special arrangement for servers
-                    if settingsdir == "servers":
-                        if option == "username" or option == "password" or option == "proxy_username" or option == "proxy_password" or option == "autologin_key":
-                            value = self.Obfuscate(self.__dict__[settingsdir][s].__dict__[option])
-                            if option == "password":
-                                if self.__dict__[settingsdir][s].save_password == "False":
-                                    value = ""
-                                elif keyring_available:
-                                    if self.__dict__[settingsdir][s].password != "":
+
+        # only import keyring lib if configured to do so - to avoid Windows crashes
+        # like https://github.com/HenriWahl/Nagstamon/issues/97
+        if self.use_system_keyring == True:
+            self.keyring_available = self.KeyringAvailable()
+
+        # one section for each setting
+        for s in self.__dict__[settingsdir]:
+            # depending on python version allow_no_value is allowed or not
+            if sys.version_info[0] < 3 and sys.version_info[1] < 7:
+                config = ConfigParser.ConfigParser()
+            else:
+                config = ConfigParser.ConfigParser(allow_no_value=True)
+            config.add_section(setting + "_" + s)
+            for option in self.__dict__[settingsdir][s].__dict__:
+                # obfuscate certain entries in config file - special arrangement for servers
+                if settingsdir == "servers":
+                    #if option == "username" or option == "password" or option == "proxy_username" or option == "proxy_password" or option == "autologin_key":
+                    if option in ["username", "password", "proxy_username", "proxy_password", "autologin_key"]:
+                        value = self.Obfuscate(self.__dict__[settingsdir][s].__dict__[option])
+                        if option == "password":
+                            if self.__dict__[settingsdir][s].save_password == "False":
+                                value = ""
+                            elif self.keyring_available and self.use_system_keyring:
+                                if self.__dict__[settingsdir][s].password != "":
+                                    # necessary to import on-the-fly due to possible Windows crashes
+                                    try:
+                                        import keyring
+                                    except:
+                                        import Nagstamon.thirdparty.keyring as keyring
+                                    # provoke crash if password saving does not work - this is the case
+                                    # on newer Ubuntu releases
+                                    try:
                                         keyring.set_password("Nagstamon", "@".join((self.__dict__[settingsdir][s].username,
-                                                                                    self.__dict__[settingsdir][s].monitor_url)),
-                                                                                    self.__dict__[settingsdir][s].password)
-                                    value = ""
-                            if option == "proxy_password":
-                                if keyring_available:
-                                    if self.__dict__[settingsdir][s].proxy_password != "":
+                                                                                self.__dict__[settingsdir][s].monitor_url)),
+                                                                                self.__dict__[settingsdir][s].password)
+                                    except:
+                                        import traceback
+                                        traceback.print_exc(file=sys.stdout)
+                                        sys.exit(1)
+                                value = ""
+                        if option == "proxy_password":
+                            if self.keyring_available and self.use_system_keyring:
+                                # necessary to import on-the-fly due to possible Windows crashes
+                                try:
+                                    import keyring
+                                except:
+                                    import Nagstamon.thirdparty.keyring as keyring
+                                if self.__dict__[settingsdir][s].proxy_password != "":
+                                    # provoke crash if password saving does not work - this is the case
+                                    # on newer Ubuntu releases
+                                    try:
                                         keyring.set_password("Nagstamon", "@".join(("proxy",\
-                                                                                    self.__dict__[settingsdir][s].proxy_username,
-                                                                                    self.__dict__[settingsdir][s].proxy_address)),
-                                                                                    self.__dict__[settingsdir][s].proxy_password)
-                                    value = ""
-                            config.set(setting + "_" + s, option, value)
-                        else:
-                            config.set(setting + "_" + s, option, self.__dict__[settingsdir][s].__dict__[option])
+                                                                                self.__dict__[settingsdir][s].proxy_username,
+                                                                                self.__dict__[settingsdir][s].proxy_address)),
+                                                                                self.__dict__[settingsdir][s].proxy_password)
+                                    except:
+                                        import traceback
+                                        traceback.print_exc(file=sys.stdout)
+                                        sys.exit(1)
+
+                                value = ""
+                        config.set(setting + "_" + s, option, value)
                     else:
                         config.set(setting + "_" + s, option, self.__dict__[settingsdir][s].__dict__[option])
+                else:
+                    config.set(setting + "_" + s, option, self.__dict__[settingsdir][s].__dict__[option])
 
-                # open, save and close config_server file
-                if not os.path.exists(self.configdir + os.sep + settingsdir):
-                    os.mkdir(self.configdir + os.sep + settingsdir)
-                f = open(os.path.normpath(self.configdir + os.sep + settingsdir + os.sep + setting + "_" + s + ".conf"), "w")
-                config.write(f)
-                f.close()
+            # open, save and close config_server file
+            if not os.path.exists(self.configdir + os.sep + settingsdir):
+                os.mkdir(self.configdir + os.sep + settingsdir)
+            f = open(os.path.normpath(self.configdir + os.sep + settingsdir + os.sep + setting + "_" + s + ".conf"), "w")
+            config.write(f)
+            f.close()
 
-            # clean up old deleted/renamed config files
-            if os.path.exists(self.configdir + os.sep + settingsdir):
-                for f in os.listdir(self.configdir + os.sep + settingsdir):
-                    if not f.split(setting + "_")[1].split(".conf")[0] in self.__dict__[settingsdir]:
-                        os.unlink(self.configdir + os.sep + settingsdir + os.sep + f)
-
-        except:
-            import traceback
-            traceback.print_exc(file=stdout)
+        # clean up old deleted/renamed config files
+        if os.path.exists(self.configdir + os.sep + settingsdir):
+            for f in os.listdir(self.configdir + os.sep + settingsdir):
+                if not f.split(setting + "_")[1].split(".conf")[0] in self.__dict__[settingsdir]:
+                    os.unlink(self.configdir + os.sep + settingsdir + os.sep + f)
 
 
     def Convert_Conf_to_Multiple_Servers(self):
@@ -598,6 +661,37 @@ class Config(object):
             self.__dict__.pop("use_proxy_no")
 
 
+    def KeyringAvailable(self):
+        """
+            determine if keyring module and an implementation is available for secure password storage
+        """
+        try:
+            # Linux systems should use keyring only if it comes with the distro, otherwise chances are small
+            # that keyring works at all
+            if not platform.system() in ["Windows", "Darwin"]:
+                # keyring and secretstorage have to be importable
+                import keyring, secretstorage
+                if ("SecretService") in dir(keyring.backends) and not (keyring.get_keyring() is None):
+                    return True
+            else:
+                # safety first - if not yet available disable it
+                if not self.__dict__.has_key("use_system_keyring"):
+                    self.use_system_keyring = False
+                # only import keyring lib if configured to do so
+                # necessary to avoid Windows crashes like https://github.com/HenriWahl/Nagstamon/issues/97
+                if self.use_system_keyring == True:
+                    # hint for packaging: nagstamon.spec always have to match module path
+                    # keyring has to be bound to object to be used later
+                    import Nagstamon.thirdparty.keyring as keyring
+                    return  not (keyring.get_keyring() is None)
+                else:
+                    return False
+        except:
+            import traceback
+            traceback.print_exc(file=sys.stdout)
+            return False
+
+
     def Convert_Conf_to_Custom_Actions(self):
         """
         any nagstamon minor to 0.9.9 will have extra ssh/rdp/vnc settings
@@ -720,7 +814,7 @@ class Config(object):
                                                     description="Acknowledges a host with one click.",
                                                     filter_target_host=False, enabled=False,
                                                     string="$MONITOR$/view.py?_transid=$TRANSID$&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=service&host=$HOST$&_ack_comment=$COMMENT-ACK$&_acknowledge=Acknowledge&service=$SERVICE$")
-        defaultactions["Check_MK Edit host in WATO"] = Action(name="Check_MK Edit host in WATO", enabled=True,
+        defaultactions["Check_MK Edit host in WATO"] = Action(name="Check_MK Edit host in WATO", enabled=False,
                                                      monitor_type="Check_MK Multisite",
                                                      description="Edit host in WATO.",
                                                      string="$MONITOR$/index.py?start_url=%2Fmonitor%2Fcheck_mk%2Fwato.py%3Fmode%3Dedithost%26host%3D$HOST$")
