@@ -1,4 +1,3 @@
-#!/usr/local/bin/python2.7
 # -*- encoding: utf-8; py-indent-offset: 4 -*-
 # +------------------------------------------------------------------+
 # |             ____ _               _        __  __ _  __           |
@@ -29,9 +28,8 @@
 import sys
 import urllib
 import webbrowser
-import traceback
-import base64
 import time
+import copy
 
 from Nagstamon import Actions
 from Nagstamon.Objects import *
@@ -57,14 +55,17 @@ class LastCheckColumnMultisite(Column):
     @classmethod
     def sort_function(cls, model, iter1, iter2, column):
         """ Overrides default sorting behaviour """
+
         data1, data2 = [model.get_value(x, column) for x in (iter1, iter2)]
+
         try:
             first = Actions.MachineSortableDateMultisite(data1)
             second = Actions.MachineSortableDateMultisite(data2)
         except ValueError, err:
             print err
             return cmp(first, second)
-        return first - second
+        # other order than default function
+        return second - first
 
 
 class DurationColumnMultisite(CustomSortingColumn):
@@ -80,7 +81,8 @@ class DurationColumnMultisite(CustomSortingColumn):
         except ValueError, err:
             print err
             return cmp(first, second)
-        return first - second
+        # other order than default function
+        return second - first
 
 
 class MultisiteServer(GenericServer):
@@ -91,13 +93,19 @@ class MultisiteServer(GenericServer):
 
     # URLs for browser shortlinks/buttons on popup window
     BROWSER_URLS= { "monitor": "$MONITOR$",\
-                    "hosts": "$MONITOR$/index.py?start_url=view.py?view_name=nagstamon_hosts",\
-                    "services": "$MONITOR$/index.py?start_url=view.py?view_name=nagstamon_svc",\
+                    "hosts": "$MONITOR$/index.py?start_url=view.py?view_name=hostproblems",\
+                    "services": "$MONITOR$/index.py?start_url=view.py?view_name=svcproblems",\
                     "history": '$MONITOR$/index.py?start_url=view.py?view_name=events'}
 
     # A Monitor CGI URL is not necessary so hide it in settings
     # autologin is used only by Centreon
-    DISABLED_CONTROLS = ["label_monitor_cgi_url", "input_entry_monitor_cgi_url", "input_checkbutton_use_autologin", "label_autologin_key", "input_entry_autologin_key"]
+    DISABLED_CONTROLS = ["label_monitor_cgi_url",
+                         "input_entry_monitor_cgi_url",
+                         "input_checkbutton_use_autologin",
+                         "label_autologin_key",
+                         "input_entry_autologin_key",
+                         "input_checkbutton_use_display_name_host",
+                         "input_checkbutton_use_display_name_service"]
 
     COLUMNS = [
         HostColumn,
@@ -132,23 +140,29 @@ class MultisiteServer(GenericServer):
         # Prepare all urls needed by nagstamon if not yet done
         if len(self.urls) == len(self.statemap):
             self.urls = {
-              'api_services':    self.monitor_url + "view.py?view_name=nagstamon_svc&output_format=python",
+              'api_services':    self.monitor_url + "view.py?view_name=nagstamon_svc&output_format=python&lang=&limit=hard",
               'human_services':  self.monitor_url + "index.py?%s" % \
                                                    urllib.urlencode({'start_url': 'view.py?view_name=nagstamon_svc'}),
               'human_service':   self.monitor_url + "index.py?%s" %
                                                    urllib.urlencode({'start_url': 'view.py?view_name=service'}),
 
-              'api_hosts':       self.monitor_url + "view.py?view_name=nagstamon_hosts&output_format=python",
+              'api_hosts':       self.monitor_url + "view.py?view_name=nagstamon_hosts&output_format=python&lang=&limit=hard",
               'human_hosts':     self.monitor_url + "index.py?%s" %
                                                    urllib.urlencode({'start_url': 'view.py?view_name=nagstamon_hosts'}),
               'human_host':      self.monitor_url + "index.py?%s" %
                                                    urllib.urlencode({'start_url': 'view.py?view_name=hoststatus'}),
-
-              'api_reschedule':  self.monitor_url + 'nagios_action.py?action=reschedule',
-              'api_host_act':    self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=hoststatus',
-              'api_service_act': self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=service',
+              # URLs do not need pythonic output because since werk #0766 API does not work with transid=-1 anymore
+              # thus access to normal webinterface is used
+              #'api_host_act':    self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=hoststatus&filled_in=actions&lang=',
+              #'api_service_act': self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=service&filled_in=actions&lang=',
+              #'api_svcprob_act': self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=svcproblems&filled_in=actions&lang=',
+              'api_host_act':    self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&view_name=hoststatus&filled_in=actions&lang=',
+              'api_service_act': self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&view_name=service&filled_in=actions&lang=',
+              'api_svcprob_act': self.monitor_url + 'view.py?_transid=-1&_do_actions=yes&_do_confirm=Yes!&view_name=svcproblems&filled_in=actions&lang=',
               'human_events':    self.monitor_url + "index.py?%s" %
                                                    urllib.urlencode({'start_url': 'view.py?view_name=events'}),
+              'togglevisibility':self.monitor_url + "user_profile.py",
+              'transid':         self.monitor_url + "view.py?actions=yes&filled_in=actions&host=$HOST$&service=$SERVICE$&view_name=service"
             }
 
             self.statemap = {
@@ -162,20 +176,8 @@ class MultisiteServer(GenericServer):
         if self.CookieAuth:
             # get cookie to access Check_MK web interface
             if len(self.Cookie) == 0:
-                # put all necessary data into url string
-                logindata = urllib.urlencode({"_username":self.get_username(),\
-                                 "_password":self.get_password(),\
-                                 "_login":"1",\
-                                 "_origtarget": "",\
-                                 "filled_in":"login"})
-
-                # get cookie from login page via url retrieving as with other urls
-                try:
-                    # login and get cookie
-                    urlcontent = self.urlopener.open(self.monitor_url + "/login.py", logindata)
-                    urlcontent.close()
-                except:
-                    self.Error(sys.exc_info())
+                # if no cookie yet login
+                self._get_cookie_login()
 
         GenericServer.init_HTTP(self)
 
@@ -208,18 +210,49 @@ class MultisiteServer(GenericServer):
             raise MultisiteError(True, Result(result = content,
                                                error = content))
 
-        # looks like cookieauth
+        # in case of auth problem enable GUI auth part in popup
+        if self.CookieAuth == True and len(self.Cookie) == 0:
+            self.refresh_authentication = True
+            return Result(result="", error="Authentication failed")
+
+       # looks like cookieauth
         elif content.startswith('<'):
             self.CookieAuth = True
-            return ""
+            # if first attempt login and then try to get data again
+            if len(self.Cookie) == 0:
+                self._get_cookie_login()
+                result = self.FetchURL(url, 'raw')
+                content, error = result.result, result.error
+                if content.startswith('<'):
+                    return ""
 
         return eval(content)
 
 
+    def _get_cookie_login(self):
+        """
+        login on cookie monitor site
+        """
+        # put all necessary data into url string
+        logindata = urllib.urlencode({"_username":self.get_username(),\
+                         "_password":self.get_password(),\
+                         "_login":"1",\
+                         "_origtarget": "",\
+                         "filled_in":"login"})
+        # get cookie from login page via url retrieving as with other urls
+        try:
+            # login and get cookie
+            urlcontent = self.urlopener.open(self.monitor_url + "/login.py", logindata)
+            urlcontent.close()
+        except:
+                    self.Error(sys.exc_info())
+
+
     def _get_status(self):
         """
         Get status from Check_MK Server
         """
+
         ret = Result()
 
         # Create URLs for the configured filters
@@ -256,12 +289,13 @@ class MultisiteServer(GenericServer):
                     new_host = n["host"]
                     self.new_hosts[new_host] = GenericHost()
                     self.new_hosts[new_host].name = n["host"]
+                    self.new_hosts[new_host].server = self.name
                     self.new_hosts[new_host].status = n["status"]
                     self.new_hosts[new_host].last_check = n["last_check"]
                     self.new_hosts[new_host].duration = n["duration"]
                     self.new_hosts[new_host].attempt = n["attempt"]
-                    self.new_hosts[new_host].status_information= n["status_information"]
-                    self.new_hosts[new_host].site    = n["site"]
+                    self.new_hosts[new_host].status_information= n["status_information"].replace("\n", " ")
+                    self.new_hosts[new_host].site = n["site"]
                     self.new_hosts[new_host].address = n["address"]
                     # transisition to Check_MK 1.1.10p2
                     if host.has_key('host_in_downtime'):
@@ -323,21 +357,22 @@ class MultisiteServer(GenericServer):
                 # host objects contain service objects
                 if not self.new_hosts.has_key(n["host"]):
                     self.new_hosts[n["host"]] = GenericHost()
-                    self.new_hosts[n["host"]].name    = n["host"]
-                    self.new_hosts[n["host"]].status  = "UP"
-                    self.new_hosts[n["host"]].site    = n["site"]
+                    self.new_hosts[n["host"]].name = n["host"]
+                    self.new_hosts[n["host"]].status = "UP"
+                    self.new_hosts[n["host"]].site = n["site"]
                     self.new_hosts[n["host"]].address = n["address"]
                 # if a service does not exist create its object
                 if not self.new_hosts[n["host"]].services.has_key(n["service"]):
                     new_service = n["service"]
                     self.new_hosts[n["host"]].services[new_service] = GenericService()
                     self.new_hosts[n["host"]].services[new_service].host = n["host"]
+                    self.new_hosts[n["host"]].services[new_service].server = self.name
                     self.new_hosts[n["host"]].services[new_service].name = n["service"]
                     self.new_hosts[n["host"]].services[new_service].status = n["status"]
                     self.new_hosts[n["host"]].services[new_service].last_check = n["last_check"]
                     self.new_hosts[n["host"]].services[new_service].duration = n["duration"]
                     self.new_hosts[n["host"]].services[new_service].attempt = n["attempt"]
-                    self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"]
+                    self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"].replace("\n", " ").strip()
                     self.new_hosts[n["host"]].services[new_service].passiveonly = n["passiveonly"]
                     self.new_hosts[n["host"]].services[new_service].flapping = n["flapping"]
                     self.new_hosts[n["host"]].services[new_service].site = n["site"]
@@ -350,10 +385,6 @@ class MultisiteServer(GenericServer):
                     if service.has_key('svc_acknowledged'):
                         if service['svc_acknowledged'] == 'yes':
                             self.new_hosts[n["host"]].services[new_service].acknowledged = True
-                    # the following lead to wrong "passive" P sign in status window
-                    #if service.has_key('svc_is_active'):
-                    #    if service['svc_is_active'] == 'no' and not service['svc_check_command'].startswith('check_mk'):
-                    #        self.new_hosts[n["host"]].services[new_service].passiveonly = True
                     if service.has_key('svc_flapping'):
                         if service['svc_flapping'] == 'yes':
                             self.new_hosts[n["host"]].services[new_service].flapping = True
@@ -426,23 +457,6 @@ class MultisiteServer(GenericServer):
         return Result(result=address)
 
 
-    def _set_recheck(self, host, service):
-        if service != "":
-            if self.hosts[host].services[service].is_passive_only() and not service['Service check command'].startswith('check_mk'):
-                # Do not check passive only checks
-                return
-
-            if self.hosts[host].services[service].command.startswith('check_mk'):
-                if str(self.conf.debug_mode) == "True":
-                    self.Debug(server=self.get_name(), host=host, debug ="This is a passive child of Check_MK. Re-schedule the Check_MK service")
-
-                service = 'Check_MK'
-
-        result = self.FetchURL(self.urls['api_reschedule'] + '&' + urllib.urlencode({'site': self.hosts[host].site,
-                                                                                     'host': host,
-                                                                                     'service': service}), giveback='raw')
-
-
     def get_start_end(self, host):
         return time.strftime("%Y-%m-%d %H:%M"), time.strftime("%Y-%m-%d %H:%M", time.localtime(time.time() + 7200))
 
@@ -454,17 +468,22 @@ class MultisiteServer(GenericServer):
         }
         params.update(specific_params)
 
-        url = self.urls['api_host_act']
-
+        # service is now added in Actions.Action(): action_type "url-check_mk-multisite"
         if service != "":
-            params['service'] = service
             url = self.urls['api_service_act']
+        else:
+            url = self.urls['api_host_act']
 
         if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), host=host, debug ="Adding downtime: " + url + '&' + urllib.urlencode(params))
-
-        result = self.FetchURL(url + '&' + urllib.urlencode(params), giveback = 'raw')
+            self.Debug(server=self.get_name(), host=host, debug ="Submitting action: " + url + '&' + urllib.urlencode(params))
 
+        action = Actions.Action(type="url-check_mk-multisite",\
+                                string=url + '&' + urllib.urlencode(params),\
+                                conf=self.conf,\
+                                host = host,\
+                                service = service,\
+                                server=self)
+        action.run()
 
     def _set_downtime(self, host, service, author, comment, fixed, start_time, end_time, hours, minutes):
         self._action(self.hosts[host].site, host, service, {
@@ -494,3 +513,52 @@ class MultisiteServer(GenericServer):
             self._action(self.hosts[host].site, host, s, p)
 
 
+    def _set_recheck(self, host, service):
+        p = {
+            '_resched_checks':    'Reschedule active checks',
+        }
+        self._action(self.hosts[host].site, host, service, p)
+
+
+    def recheck_all(self):
+        """
+        special method for Check_MK as there is one URL for rescheduling all problems to be checked
+        """
+        params = dict()
+        params['_resched_checks'] = 'Reschedule active checks'
+        url = self.urls['api_svcprob_act']
+
+        if str(self.conf.debug_mode) == "True":
+            self.Debug(server=self.get_name(), debug ="Rechecking all action: " + url + '&' + urllib.urlencode(params))
+
+        result = self.FetchURL(url + '&' + urllib.urlencode(params), giveback = 'raw')
+
+    """
+    def ToggleVisibility(self, widget):
+        #Attempt to enable/disable visibility of all problems for user via
+        #/user_profile.py?cb_ua_force_authuser=0&cb_ua_force_authuser_webservice=0&filled_in=profile
+
+        # since werk #0766 http://mathias-kettner.de/check_mk_werks.php?werk_id=766 a real transid is needed
+        transid = self.FetchURL(self.urls["togglevisibility"], "obj").result.find(attrs={"name" : "_transid"})["value"]
+        cgi_data = urllib.urlencode({"cb_ua_force_authuser" : str(int(widget.get_active())),\
+                                     "cb_ua_force_authuser_webservice" : str(int(widget.get_active())),\
+                                     "filled_in" : "profile",\
+                                     "_transid" : transid,\
+                                     "_save" : "Save"})
+
+        self.FetchURL(self.urls["togglevisibility"], "raw", cgi_data=urllib.urlencode(\
+                                                                    {"cb_ua_force_authuser" : str(int(widget.get_active())),\
+                                                                     "cb_ua_force_authuser_webservice" : str(int(widget.get_active())),\
+                                                                     "filled_in" : "profile",\
+                                                                     "_transid" : transid,\
+                                                                     "_save" : "Save"})).result
+    """
+
+
+    def _get_transid(self, host, service):
+        """
+        get transid for an action
+        """
+        transid = self.FetchURL(self.urls["transid"].replace("$HOST$", host).replace("$SERVICE$", service.replace(" ", "+")),\
+                                "obj").result.find(attrs={"name" : "_transid"})["value"]
+        return transid
